@@ -9,13 +9,19 @@ try:
     import readline
 
     RL = True
+    UNIX = True
+    WINDOWS = False
 except ImportError:
 
     RL = False
+    WINDOWS = True
+    UNIX = False
 
 from os import chdir
 from shlex import split as shlex_split
-from typing import List
+from typing import List, Union
+
+from soco import SoCo  # type: ignore
 
 from soco_cli.action_processor import get_actions, list_actions
 from soco_cli.aliases import AliasManager
@@ -73,8 +79,15 @@ ACTIONS_TO_EXEC_NO_SPEAKER = [
 ]
 
 
+LOG_SETTING = ""
+
+
 def interactive_loop(
-    speaker_name, use_local_speaker_list=False, no_env=False, single_keystroke=False
+    speaker_name,
+    log_setting,
+    use_local_speaker_list=False,
+    no_env=False,
+    single_keystroke=False,
 ):
     """
     The main interactive loop for gathering and processing interactive commands.
@@ -82,11 +95,15 @@ def interactive_loop(
     Args:
         speaker_name (str): The name of the speaker supplied if supplied on the command
             line.
+        log_setting (str): The logging option.
         use_local_speaker_list (bool): Whether to use cached discovery.
         no_env (bool): Whether to ignore environment variables.
         single_keystroke (bool): Whether to start in single keystroke mode.
     """
 
+    global LOG_SETTING
+    LOG_SETTING = "--log=" + log_setting
+
     speaker = None
     saved_speaker = None
     pushed = False
@@ -352,12 +369,17 @@ def interactive_loop(
                     alias_name = command.pop(0)
                     if alias_name == "alias":
                         print("Not permitted: cannot create alias for 'alias'")
-                        continue
+                        break
                     if len(command) == 0:
                         if am.create_alias(alias_name, None):
                             print("Alias '{}' removed".format(alias_name))
                         else:
                             print("Alias '{}' not found".format(alias_name))
+                    if _loop_action_in_command_line(command_line):
+                        print(
+                            "Not permitted: cannot create alias containing 'loop' actions"
+                        )
+                        break
                     else:
                         # Have to collect the remaining sequences: they're all
                         # part of the alias. Reconstruction required.
@@ -386,6 +408,12 @@ def interactive_loop(
                     )
                     continue
 
+                # Check for loop statements; run in a subprocess
+                if _exec_loop_in_subprocess(
+                    speaker, command_line, use_local_speaker_list
+                ):
+                    break
+
                 # Command processing
                 try:
                     args = command
@@ -672,7 +700,6 @@ class AliasProcessor:
 
     def __init__(self):
         self._used_aliases = []
-        self._seq_number = 0
         self._recurse_level = 0
         self._command_count = 0
         self._index = 0
@@ -681,28 +708,27 @@ class AliasProcessor:
     def process(self, command, am, command_list):
 
         self._recurse_level += 1
+        alias_name = command[0]
+        alias_parms = command[1:]
+        self._command_list = command_list
+        seq_number = len(command_list)
 
         logging.info(
             "Alias unpacking: recursion level {}, sequence number {}".format(
-                self._recurse_level, self._seq_number
+                self._recurse_level, seq_number
             )
         )
 
-        alias_name = command[0]
-        alias_parms = command[1:]
-
-        self._command_list = command_list
-
         # Detect loops
         for used_alias in self._used_aliases:
             if used_alias[0] != self._recurse_level:
-                if used_alias[1] == self._seq_number and used_alias[2] == alias_name:
+                if used_alias[1] == seq_number and used_alias[2] == alias_name:
                     # Alias name reused at different recursion levels but within
                     # the unpacking of the same sequence signifies a loop.
                     print("Error: Alias loop detected ... stopping")
                     self._remove_added_commands()
                     return False
-        self._used_aliases.append((self._recurse_level, self._seq_number, alias_name))
+        self._used_aliases.append((self._recurse_level, seq_number, alias_name))
 
         alias_actions = am.action(alias_name)
         try:
@@ -718,44 +744,33 @@ class AliasProcessor:
         logging.info("Unpacking the alias '{}' -> '{}'".format(alias_name, sequences))
 
         index = command_list.index()
-        for idx, sequence in enumerate(sequences, start=1):
-            self._seq_number = idx
-
+        for sequence in sequences:
             alias_parms_local = alias_parms.copy()
-
-            # Deprecated
-            if "_" in sequence:
-                logging.info("Deprecated: Suppressing any additional parameters")
-                print("*** Warning: The use of '_' is now deprecated in favour of")
-                print("*** positional arguments (%1, %2, etc.).")
-                position = sequence.index("_")
-                sequence = sequence[:position]
-            else:
-                # Positional argument substitution: %1, %2, etc.
-                alias_parms_used = []
-                for i, item in enumerate(sequence):
-                    if item in self._arg_names:
-                        parm_index = int(item[1]) - 1
-                        try:
-                            sequence[i] = alias_parms_local[parm_index]
-                            logging.info(
-                                "Substituting '{}' for arg. {}".format(
-                                    alias_parms_local[parm_index], item
-                                )
+            # Positional argument substitution: %1, %2, etc.
+            alias_parms_used = []
+            for i, item in enumerate(sequence):
+                if item in self._arg_names:
+                    parm_index = int(item[1]) - 1
+                    try:
+                        sequence[i] = alias_parms_local[parm_index]
+                        logging.info(
+                            "Substituting '{}' for arg. {}".format(
+                                alias_parms_local[parm_index], item
                             )
-                            # This allows reuse of the same parameter
-                            alias_parms_used.append(parm_index)
-                        except IndexError:
-                            logging.info("No value found for arg. {}".format(item))
-                            sequence[i] = None
+                        )
+                        # This allows reuse of the same parameter
+                        alias_parms_used.append(parm_index)
+                    except IndexError:
+                        logging.info("No value found for arg. {}".format(item))
+                        sequence[i] = None
 
-                # Remove unsatisfied arguments and substituted parameters
-                sequence = [x for x in sequence if x is not None]
-                alias_parms_local = [
-                    y
-                    for x, y in enumerate(alias_parms_local)
-                    if not x in alias_parms_used
-                ]
+            # Remove unsatisfied arguments and substituted parameters
+            sequence = [x for x in sequence if x is not None]
+            alias_parms_local = [
+                y
+                for x, y in enumerate(alias_parms_local)
+                if not x in alias_parms_used
+            ]
 
             # Recurse if the sequence is itself an alias
             try:
@@ -779,7 +794,7 @@ class AliasProcessor:
                     index += 1
                     self._command_count += 1
                     self._index = index
-
+                    logging.info("Current command list = {}".format(command_list))
             except IndexError:
                 logging.info("Empty sequence ... returning")
                 return False
@@ -836,6 +851,9 @@ def _exec(command_args: List[str]) -> None:
     set_suspend_sighandling(suspend=False)
 
 
+CTRL_C_MSG_ISSUED = False
+
+
 def _exec_action(speaker_ip: str, action: str, args: List[str]) -> None:
     # Commands to run in a subprocess, to allow CTRL-C
     # to exit the subprocess only, and not the shell.
@@ -846,14 +864,7 @@ def _exec_action(speaker_ip: str, action: str, args: List[str]) -> None:
         command_line = [sys.argv[0], speaker_ip, action, *args]
 
     # Pass through logging option
-    for position, arg in enumerate(sys.argv[1:]):
-        if arg.startswith("--log="):
-            command_line.insert(1, arg)
-            break
-        if arg == "--log":
-            command_line.insert(1, arg)
-            command_line.append(sys.argv[1:][position + 1])
-            break
+    command_line.insert(1, LOG_SETTING)
 
     global CTRL_C_MSG_ISSUED
     if CTRL_C_MSG_ISSUED is False:
@@ -863,4 +874,78 @@ def _exec_action(speaker_ip: str, action: str, args: List[str]) -> None:
     _exec(command_line)
 
 
-CTRL_C_MSG_ISSUED = False
+def _exec_command_line(command_line: str) -> None:
+    """Runs a sonos command line as a subprocess, in its own shell.
+
+    Args:
+        command_line (str): The command line to execute.
+    """
+
+    set_suspend_sighandling(suspend=True)
+    try:
+        logging.info("Running command: '{}'".format(command_line))
+        subprocess.run(command_line, shell=True)
+    except Exception as e:
+        print(e)
+    set_suspend_sighandling(suspend=False)
+
+
+def _exec_loop_in_subprocess(
+    speaker: Union[SoCo, None], command_line: str, use_local: bool
+) -> bool:
+    """If there's a loop statement, run the command in a subprocess.
+
+    Args:
+        speaker (SoCo, None): The speaker to which the command is targeted, or
+            None if the speaker is in the command line.
+        command_line (str): The complete command line.
+        use_local (bool): use the local speaker list.
+
+    Returns:
+        bool: True if there's a loop statement, False otherwise.
+    """
+    if _loop_action_in_command_line(command_line):
+        global LOG_SETTING
+        sonos_command = "sonos " + LOG_SETTING + " "
+        if speaker is not None:
+            # This is a way of using the required speaker for each
+            # invocation in the list of commands, using the SPKR env. variable.
+            if UNIX:
+                command_line = (
+                    "export SPKR="
+                    + speaker.ip_address
+                    + " && "
+                    + sonos_command
+                    + command_line
+                )
+            elif WINDOWS:
+                command_line = (
+                    'set "SPKR='
+                    + speaker.ip_address
+                    + '" && '
+                    + sonos_command
+                    + command_line
+                )
+        else:
+            if use_local:
+                sonos_command = sonos_command + "-l "
+            command_line = sonos_command + command_line
+        logging.info("'loop' statement found, command line = '{}'".format(command_line))
+        _exec_command_line(command_line)
+        return True
+
+    return False
+
+
+def _loop_action_in_command_line(command_line: str) -> bool:
+    """Does the command line contain a loop statement
+
+    Args:
+        command_line (str): The command line to test.
+
+    Returns:
+        bool: True if there's a loop in this command line.
+    """
+    return any(
+        word in command_line.split() for word in ["loop", "loop_until", "loop_for"]
+    )
